"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactIntl = require("react-intl");

var _Select = require("./Select");

var _TextField = _interopRequireDefault(require("./TextField"));

var _DateField = _interopRequireDefault(require("./DateField"));

var _DateTimeField = _interopRequireDefault(require("./DateTimeField"));

var _strings = require("../utils/strings");

var _FormattedFieldname = _interopRequireDefault(require("../internal/FormattedFieldname"));

// @flow
var Field = function Field(_ref) {
  var field = _ref.field,
      value = _ref.value,
      onChange = _ref.onChange;

  var _useIntl = (0, _reactIntl.useIntl)(),
      locale = _useIntl.locale;

  var label = _react["default"].createElement(_FormattedFieldname["default"], {
    field: field
  });

  var placeholder
  /*: string*/
  = (0, _strings.getLocalizedFieldProp)(field, 'placeholder', locale);

  var handleChange = function handleChange(newValue) {
    onChange(field.key, newValue);
  };

  switch (field.type) {
    case 'text':
    case 'textarea':
      return _react["default"].createElement(_TextField["default"], {
        value: value,
        multiline: true,
        onChange: handleChange,
        label: label,
        placeholder: placeholder
      });

    case 'select_one':
      return _react["default"].createElement(_Select.SelectOne, {
        value: value,
        options: field.options,
        onChange: handleChange,
        label: label,
        placeholder: placeholder
      });

    case 'select_multiple':
      return _react["default"].createElement(_Select.SelectMultiple, {
        value: value,
        label: label
      });

    case 'number':
      return _react["default"].createElement(_TextField["default"], {
        value: value,
        onChange: handleChange,
        label: label,
        type: "number",
        placeholder: placeholder
      });

    case 'date':
      return _react["default"].createElement(_DateField["default"], {
        value: value,
        onChange: handleChange,
        label: label,
        placeholder: placeholder
      });

    case 'datetime':
      return _react["default"].createElement(_DateTimeField["default"], {
        value: value,
        onChange: handleChange,
        label: label,
        placeholder: placeholder
      });

    default:
      return _react["default"].createElement(_TextField["default"], {
        value: value,
        disabled: true,
        multiline: true,
        label: label,
        placeholder: placeholder
      });
  }
};

var _default = Field;
exports["default"] = _default;
//# sourceMappingURL=Field.js.map