"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

require("core-js/modules/es.array.join");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.SelectMultiple = exports.SelectOne = void 0;

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _find = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _extends2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/extends"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/objectWithoutProperties"));

var _react = _interopRequireWildcard(require("react"));

var _downshift = _interopRequireDefault(require("downshift"));

var _TextField = _interopRequireDefault(require("./TextField"));

var _Popper = _interopRequireDefault(require("@material-ui/core/Popper"));

var _Paper = _interopRequireDefault(require("@material-ui/core/Paper"));

var _MenuItem = _interopRequireDefault(require("@material-ui/core/MenuItem"));

var _matchSorter = _interopRequireDefault(require("match-sorter"));

var _reactIntl = require("react-intl");

var _styles = require("@material-ui/core/styles");

// @flow
var m = (0, _reactIntl.defineMessages)({
  yes: {
    "id": "KLjFTA==",
    "defaultMessage": 'Yes'
  },
  no: {
    "id": "U0HM2A==",
    "defaultMessage": 'No'
  }
});
var useStyles = (0, _styles.makeStyles)(function (theme) {
  return {
    root: {
      flexGrow: 1,
      height: 250
    },
    container: {
      width: '100%'
    },
    paper: {
      position: 'absolute',
      zIndex: 1,
      marginTop: theme.spacing(1),
      left: 0,
      right: 0
    },
    chip: {
      margin: theme.spacing(0.5, 0.25)
    },
    inputRoot: {
      flexWrap: 'wrap'
    },
    inputInput: {
      width: 'auto',
      flexGrow: 1
    },
    divider: {
      height: theme.spacing(2)
    }
  };
});

function renderInput(inputProps) {
  var InputProps = inputProps.InputProps,
      classes = inputProps.classes,
      ref = inputProps.ref,
      other = (0, _objectWithoutProperties2["default"])(inputProps, ["InputProps", "classes", "ref"]);
  return _react["default"].createElement(_TextField["default"], (0, _extends2["default"])({
    InputProps: (0, _assign["default"])({
      inputRef: ref,
      classes: {
        root: classes.inputRoot,
        input: classes.inputInput
      }
    }, InputProps)
  }, other));
}
/*:: type Suggestion = {|
  label: string,
  value: SelectableFieldValue
|}*/

/*:: type RenderSuggestionProps = {|
  highlightedIndex: number | null,
  index: number,
  itemProps: any,
  selectedItem: string,
  suggestion: Suggestion
|}*/


function renderSuggestion(suggestionProps
/*: RenderSuggestionProps*/
) {
  var _context;

  var suggestion = suggestionProps.suggestion,
      index = suggestionProps.index,
      itemProps = suggestionProps.itemProps,
      highlightedIndex = suggestionProps.highlightedIndex,
      selectedItem = suggestionProps.selectedItem;
  var isHighlighted = highlightedIndex === index;
  var isSelected = (0, _indexOf["default"])(_context = selectedItem || '').call(_context, suggestion.label) > -1;
  return _react["default"].createElement(_MenuItem["default"], (0, _extends2["default"])({}, itemProps, {
    key: suggestion.label,
    selected: isHighlighted,
    component: "div",
    style: {
      fontWeight: isSelected ? 500 : 400,
      textOverflow: 'ellipsis'
    }
  }), suggestion.label);
}

function getSuggestions(value
/*: string*/
, suggestions
/*: Array<Suggestion>*/
)
/*: Array<Suggestion>*/
{
  if (value.length === 0) return suggestions;
  var matched = (0, _matchSorter["default"])(suggestions, value, {
    keys: [function (item) {
      return item.label;
    }, function (item) {
      return typeof item.value === 'string' ? item.value : undefined;
    }]
  });
  if (matched && matched.length === 1 && matched[0].value === value) return [];
  return matched;
}
/*:: type Props = {
  value: SelectableFieldValue,
  placeholder?: string,
  onChange: (value: SelectableFieldValue) => any,
  options: SelectOptions
}*/


/**
 * A multi-select field that allows the user to enter a value that is not on the
 * list. Allows the selection of non-string values from a list, but the labels
 * for each item must be unique for this to work reliably
 */
var SelectOne = function SelectOne(_ref) {
  var value = _ref.value,
      placeholder = _ref.placeholder,
      onChange = _ref.onChange,
      options = _ref.options,
      otherProps = (0, _objectWithoutProperties2["default"])(_ref, ["value", "placeholder", "onChange", "options"]);
  var popperNode = (0, _react.useRef)();
  var classes = useStyles();

  var _useIntl = (0, _reactIntl.useIntl)(),
      t = _useIntl.formatMessage;

  var suggestions
  /*: Array<Suggestion>*/
  = (0, _react.useMemo)(function () {
    var _context2;

    return (0, _sort["default"])(_context2 = (0, _map["default"])(options).call(options, function (opt) {
      if (opt === true) return {
        value: true,
        label: t(m.yes)
      };
      if (opt === false) return {
        value: false,
        label: t(m.no)
      };
      if (typeof opt === 'number') return {
        value: opt,
        label: opt + ''
      };
      if (typeof opt === 'string') return {
        value: opt,
        label: opt
      }; // TODO: not sure how to handle this - this is "no answer" I think?

      if (opt === null) return {
        value: null,
        label: ''
      };
      return opt;
    })).call(_context2, function (a, b) {
      return a.label.localeCompare(b.label);
    });
  }, [options, t]);
  var matchingSuggestion = (0, _find["default"])(suggestions).call(suggestions, function (item) {
    return lowerCaseEqual(item.value, value);
  });
  var displayValue = matchingSuggestion ? matchingSuggestion.label : value;

  function onStateChange(changes) {
    var newValue;

    if (Object.prototype.hasOwnProperty.call(changes, 'selectedItem')) {
      newValue = changes.selectedItem;
    } else if (Object.prototype.hasOwnProperty.call(changes, 'inputValue')) {
      newValue = changes.inputValue;
    } else {
      return;
    }

    var matchingSuggestion = (0, _find["default"])(suggestions).call(suggestions, function (item) {
      return lowerCaseEqual(item.label === newValue);
    });
    newValue = newValue === undefined ? null : newValue;
    onChange(matchingSuggestion ? matchingSuggestion.value : newValue);
  }

  return _react["default"].createElement(_downshift["default"], {
    id: "downshift-popper",
    selectedItem: typeof displayValue === 'string' ? displayValue : '',
    onStateChange: onStateChange
  }, function (_ref2) {
    var _context3;

    var clearSelection = _ref2.clearSelection,
        getInputProps = _ref2.getInputProps,
        getItemProps = _ref2.getItemProps,
        getLabelProps = _ref2.getLabelProps,
        getMenuProps = _ref2.getMenuProps,
        highlightedIndex = _ref2.highlightedIndex,
        inputValue = _ref2.inputValue,
        isOpen = _ref2.isOpen,
        openMenu = _ref2.openMenu,
        selectedItem = _ref2.selectedItem;

    var _getInputProps = getInputProps({
      onChange: function onChange(event) {
        if (event.target.value === '') {
          clearSelection();
        }
      },
      onFocus: openMenu,
      placeholder: placeholder
    }),
        onBlur = _getInputProps.onBlur,
        onFocus = _getInputProps.onFocus,
        inputProps = (0, _objectWithoutProperties2["default"])(_getInputProps, ["onBlur", "onFocus"]);

    return _react["default"].createElement("div", {
      className: classes.container
    }, renderInput((0, _assign["default"])({
      fullWidth: true,
      classes: classes,
      InputProps: {
        onBlur: onBlur,
        onFocus: onFocus
      },
      InputLabelProps: getLabelProps({
        shrink: true
      }),
      inputProps: inputProps,
      ref: function ref(node) {
        popperNode.current = node;
      }
    }, otherProps)), _react["default"].createElement(_Popper["default"], {
      open: isOpen,
      anchorEl: popperNode.current,
      style: {
        zIndex: 9999
      }
    }, _react["default"].createElement("div", isOpen ? getMenuProps({}, {
      suppressRefError: true
    }) : {}, _react["default"].createElement(_Paper["default"], {
      square: true,
      style: {
        // TODO: When the popover is rendered above the input, when
        // the list is less than 400px, the bottom is no longer
        // aligned with the input field
        marginTop: 8,
        width: popperNode.current ? popperNode.current.clientWidth : undefined,
        maxHeight: 400,
        overflow: 'scroll'
      }
    }, (0, _map["default"])(_context3 = getSuggestions(inputValue, suggestions)).call(_context3, function (suggestion, index) {
      return renderSuggestion({
        suggestion: suggestion,
        index: index,
        itemProps: getItemProps({
          item: suggestion.label
        }),
        highlightedIndex: highlightedIndex,
        selectedItem: selectedItem
      });
    })))));
  });
};

exports.SelectOne = SelectOne;

var SelectMultiple = function SelectMultiple(_ref3) {
  var value = _ref3.value,
      label = _ref3.label;
  return _react["default"].createElement(_TextField["default"], {
    value: (0, _isArray["default"])(value) ? value.join(', ') : value,
    label: label,
    disabled: true
  });
}; // for two values, if strings, compare lower case, otherwise strict equal


exports.SelectMultiple = SelectMultiple;

function lowerCaseEqual(a
/*: any*/
, b
/*: any*/
) {
  if (typeof a === 'string' && typeof b === 'string') {
    return a.toLowerCase() === b.toLowerCase();
  } else {
    return a === b;
  }
}
//# sourceMappingURL=Select.js.map