"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.flatObjectEntries = flatObjectEntries;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _getIterator2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/get-iterator"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _helpers = require("./helpers");

// @flow
function isArrayOfPrimitives(value
/*: any*/
)
/*: boolean*/
{
  return (0, _isArray["default"])(value) && value.length && !(0, _some["default"])(value).call(value, _helpers.isObj);
}
/*:: type Primitive = string | boolean | null | void | number*/


/**
 * Similar to Object.entries, but flattens an object and keys are arrays. Arrays
 * of primitives are *not* flattened.
 *
 * @example
 * flatObjectEntries({
 *   foo: { bar: 'qux' },
 *   hello: 'world',
 *   arr: ['biz', 'baz'],
 *   other: [{ foo: 'bar'}]
 * })
 * // [
 * //   [['foo', 'bar'], 'qux'],
 * //   [['hello'], 'world'],
 * //   [['arr'], ['biz', 'baz']],
 * //   [['other', 0, 'foo'], 'bar']
 * // ]
 */
function flatObjectEntries(object
/*: {}*/
)
/*: Array<
  [Array<string | number>, Primitive | Primitive[]]
>*/
{
  var entries = [];

  (function step(object
  /*: any*/
  ) {
    var prev
    /*: Array<string | number>*/
    = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var currentDepth
    /*: number*/
    = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
    var keys = (0, _isArray["default"])(object) ? (0, _map["default"])(object).call(object, function (_, i) {
      return i;
    }) : (0, _keys["default"])(object);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator2["default"])(keys), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var key = _step.value;
        var value = object[key];
        var newKey = (0, _concat["default"])(prev).call(prev, [key]);

        if ((0, _helpers.isObj)(value) && !isArrayOfPrimitives(value)) {
          step(value, newKey, currentDepth + 1);
        } else {
          entries.push([newKey, value]);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator["return"] != null) {
          _iterator["return"]();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  })(object);

  return entries;
}
//# sourceMappingURL=flat_object_entries.js.map