import "core-js/modules/es.array.iterator";
import "core-js/modules/web.dom-collections.iterator";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _setTimeout from "@babel/runtime-corejs3/core-js-stable/set-timeout";
import _sliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/slice";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { withStyles } from '@material-ui/core/styles';
import Chip from '@material-ui/core/Chip';
import Select from './Select';
const styleSheet = {
  row: {
    display: 'flex',
    flexWrap: 'wrap',
    marginTop: -3
  },
  chip: {
    margin: '3px 3px 0 0',
    height: 30
  },
  select: {
    minWidth: 220
  },
  highlight: {
    backgroundColor: '#ff9696'
  }
};

class MultiSelect extends Component {
  constructor(...args) {
    var _context;

    super(...args);
    this.state = {
      selectValue: '',
      suggestions: _filterInstanceProperty(_context = this.props.suggestions).call(_context, s => {
        var _context2;

        return _indexOfInstanceProperty(_context2 = this.props.value).call(_context2, s) === -1;
      })
    };

    this.handleSelectChange = (e, d) => {
      this.setState({
        selectValue: d.newValue
      });
    };

    this.handleKeyDown = e => {
      var _context3;

      if (this.justAdded || e.keyCode !== 13) {
        this.justAdded = false;
        return;
      }

      const newValue = this.state.selectValue;

      const index = _indexOfInstanceProperty(_context3 = this.props.value).call(_context3, newValue);

      if (index > -1) {
        this.highlightChip(index);
        this.setState({
          selectValue: ''
        });
      } else {
        this.handleSuggestionSelected(e, {
          suggestionValue: newValue
        });
      }
    };

    this.handleSuggestionSelected = (e, d) => {
      var _context4;

      const newValue = _concatInstanceProperty(_context4 = this.props.value).call(_context4, d.suggestionValue);

      this.justAdded = true;
      this.setState({
        selectValue: ''
      });
      this.props.onChange(e, {
        newValue,
        type: 'add'
      });
    };

    this.handleRequestDelete = i => {
      return e => {
        var _context5;

        const {
          value
        } = this.props;

        const newValue = _concatInstanceProperty(_context5 = _sliceInstanceProperty(value).call(value, 0, i)).call(_context5, _sliceInstanceProperty(value).call(value, i + 1));

        this.props.onChange(e, {
          newValue,
          type: 'delete'
        });
      };
    };
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.value !== this.props.value) {
      var _context6;

      const suggestions = _filterInstanceProperty(_context6 = nextProps.suggestions).call(_context6, s => {
        var _context7;

        return _indexOfInstanceProperty(_context7 = nextProps.value).call(_context7, s) === -1;
      });

      this.setState({
        suggestions
      });
    }
  }

  highlightChip(i) {
    this.setState({
      highlight: i
    });

    _setTimeout(() => this.setState({
      highlight: null
    }), 500);
  }

  render() {
    const {
      classes,
      value
    } = this.props;
    return React.createElement("div", {
      className: classes.row
    }, _mapInstanceProperty(value).call(value, (d, i) => React.createElement(Chip, {
      label: d,
      key: i,
      onRequestDelete: this.handleRequestDelete(i),
      className: classes.chip + (this.state.highlight === i ? ' ' + classes.highlight : '')
    })), React.createElement(Select, {
      value: this.state.selectValue,
      suggestions: this.state.suggestions,
      onChange: this.handleSelectChange,
      onKeyDown: this.handleKeyDown,
      onSuggestionSelected: this.handleSuggestionSelected,
      className: classes.select
    }));
  }

}

MultiSelect.propTypes = {
  classes: PropTypes.object.isRequired,
  className: PropTypes.string,
  value: PropTypes.array
};
MultiSelect.defaultProps = {
  value: []
};
export default withStyles(styleSheet)(MultiSelect);
//# sourceMappingURL=MultiSelect.js.map